/* -------------------------------------------------------------------------- */
.BoxTable {
        width:          90%;
}

.BoxTable>td {
        width:          100%;
}

/* -------------------------------------------------------------------------- */
.ParamTable {
        text-align:     right;
}

.ParamTable td.Desc {
        font-weight:    bold;
        text-align:     right;
}

/* -------------------------------------------------------------------------- */
.ReportTable {
	font:		Icon;
        width:          100%;
	border:		1px Solid ThreeDShadow;
	background:	Window;
	color:		WindowText;
        border-spacing: 0px;
}

.ReportTable thead {
        background:     ButtonFace;
        font-weight:    bold; 
}

.ReportTable th {
        border:         1px solid; 
        border-color:   ButtonHighlight ButtonShadow
                        ButtonShadow ButtonHighlight;
        cursor:         default; 
        text-align:     center; 
	padding:        1px 1px 1px 1px; 
}

.ReportTable th:active {
	border-color:   ButtonShadow ButtonHighlight 
                        ButtonHighlight ButtonShadow;
	padding:        1px 1px 1px 1px; 
}

.ReportTable tbody tr.OddRow {
        background:     #FFFFFF
}

.ReportTable tbody tr.EvenRow {
        background:     #CCDBEA
}

.ReportTable tfoot {
        background:     #FFFFFF;
        font-weight:    bold; 
}

.ReportTable tfoot tr {
	border:		1px Solid ThreeDShadow;
}

.ReportTable>td {
	padding:	2px 5px;
}

.ReportTable td.Total {
        text-align:     right;
}

/* -------------------------------------------------------------------------- */
.SortArrow {
	width:          11px;
	height:         11px;
	background-position: center center;
	background-repeat: no-repeat;
	margin:         0 2px;
}

.SortArrow.descending {
	background-image: url("descending.png"); /* relative to this css file */
}

.SortArrow.ascending {
	background-image: url("ascending.png");  /* relative to this css file */
}
