set Xmp.MP.RegionInfo/MPRI:Regions
set Xmp.MP.RegionInfo/MPRI:Regions[1]/MPReg:Rectangle 0.11, 0.22, 0.33, 0.44
set Xmp.MP.RegionInfo/MPRI:Regions[1]/MPReg:PersonDisplayName Baby Gnu
set Xmp.mwg-rs.Regions/mwg-rs:AppliedToDimensions/stDim:w 1600
set Xmp.mwg-rs.Regions/mwg-rs:AppliedToDimensions/stDim:h 800
set Xmp.mwg-rs.Regions/mwg-rs:AppliedToDimensions/stDim:unit pixel
set Xmp.mwg-rs.Regions/mwg-rs:RegionList
set Xmp.mwg-rs.Regions/mwg-rs:RegionList[1]/mwg-rs:Name Baby Gnu
set Xmp.mwg-rs.Regions/mwg-rs:RegionList[1]/mwg-rs:Type Face
set Xmp.mwg-rs.Regions/mwg-rs:RegionList[1]/mwg-rs:Area/stArea:x 0.275312
set Xmp.mwg-rs.Regions/mwg-rs:RegionList[1]/mwg-rs:Area/stArea:y 0.3775
set Xmp.mwg-rs.Regions/mwg-rs:RegionList[1]/mwg-rs:Area/stArea:w 0.164375
set Xmp.mwg-rs.Regions/mwg-rs:RegionList[1]/mwg-rs:Area/stArea:h 0.28125
set Xmp.mwg-rs.Regions/mwg-rs:RegionList[1]/mwg-rs:Area/stArea:unit normalized
